#define WINDOW_TEXT              "Base Program"                                              
#define WINDOW_RESOLUTION_X      800                                                 
#define WINDOW_RESOLUTION_Y      600                                                 
#define COLOR_DEPTH              32                                                        
#define FULLSCREEN               false                                                      
#define CONFIG_FILE              "Video_Settings.cfg"

////////////////////////////////////////////////////////////////////////////////
//Name:    Resolution
//Purpose: Linked List item that holds available resolutions
////////////////////////////////////////////////////////////////////////////////
struct resolution
{resolution *next;
 int x;
 int y;
 char display_text[13];
};

////////////////////////////////////////////////////////////////////////////////
//Name:    SDL
//Purpose: Holds all our window specific code
////////////////////////////////////////////////////////////////////////////////
class sdl
{int resolution_x;
 int resolution_y;
 int color_depth;
 bool fullscreen;
 //-Linked List-//
 resolution *first;
 resolution *last;
////////////////////////////////////////////////////////////////////////////////
//Name:    Write Config
//Purpose: Write to the config file when we change things
////////////////////////////////////////////////////////////////////////////////
 void write_config()
 {ofstream fout;
  fout.open(CONFIG_FILE);
  if(fout.is_open()==true)
    {fout<<resolution_x<<endl;
     fout<<resolution_y<<endl;
     fout<<color_depth<<endl;
     fout<<fullscreen<<endl;
    }
  else
    {printf("ERROR:  SDL.WriteConfig -> Unable to Write to File: %s\n",CONFIG_FILE);
    }
  fout.close();
 } 

////////////////////////////////////////////////////////////////////////////////
//Name:    Revert Defaults
//Purpose: Revert settings to defaults
////////////////////////////////////////////////////////////////////////////////
 void revert_defaults()
 {resolution_x=WINDOW_RESOLUTION_X;
  resolution_y=WINDOW_RESOLUTION_Y;
  color_depth=COLOR_DEPTH;
  fullscreen=FULLSCREEN;
  //--//
  printf("DEBUG:  Attempting to Init SDL Window at %i x %i, %i bit depth \n",resolution_x,resolution_y,color_depth);
  screen=SDL_SetVideoMode(resolution_x, resolution_y, color_depth, SDL_SWSURFACE);
  if(screen==NULL)
    {printf("SUPER CRITICAL EXISTANCE FAILURE: Unable to Create SDL Window with Default Values \n");
     done=true;
    }
  write_config();
 }
 
////////////////////////////////////////////////////////////////////////////////
//Name:    Read Config
//Purpose: Read the config file for settings
////////////////////////////////////////////////////////////////////////////////
 bool read_config()
 {ifstream fin;
  fin.open(CONFIG_FILE);
  if(fin.is_open()==true)
    {fin>>resolution_x;
     fin>>resolution_y;
     fin>>color_depth;
     fin>>fullscreen;
    }
  else
    {printf("ERROR:  SDL.ReadConfig -> Unable to Read From File: %s - Reverting to Defaults\n",CONFIG_FILE);
     revert_defaults();
     return false;
    }
  fin.close();
 }

////////////////////////////////////////////////////////////////////////////////
//Name:    Resolution_Add
//Purpose: Adds a resolution in our linked list for later use
////////////////////////////////////////////////////////////////////////////////
 void resolution_add(int res_x, int res_y)
 {resolution *newres  = (resolution*)malloc(sizeof(resolution));
  //--//
  newres->x = res_x;
  newres->y = res_y;
  sprintf(newres->display_text,"%i x %i",res_x, res_y);
  newres->next = NULL;
  //--//
  if(first!=NULL)
    {last->next=newres;
    }
  //--//
  else
    {first=newres;
    }
  //--//
  last=newres;
 }

////////////////////////////////////////////////////////////////////////////////
//Name:    Resolution_Check
//Purpose: Gets all fullscreen resolutions for your monitor >= 800x600
//Notes:   A lot of these aren't 4:3 or 16:9
////////////////////////////////////////////////////////////////////////////////
 void fullscreen_check()
 {SDL_Rect **modes;
  //- Get available fullscreen/software surface resolutions -//
  modes=SDL_ListModes(NULL, SDL_FULLSCREEN);
  //-There are no available resolutions for the flags set-//
  if (modes == (SDL_Rect**)0) 
     {printf("DEBUG:  No Available Fullscreen Resolutions \n");
     }
  //-There are no restrictions on the resolution-//
  else if (modes == (SDL_Rect**)-1) 
     {printf("DEBUG:  Infinite Potential Fullscreen Resolutions (How Did You Manage That?) \n");
     }
  //-This should always happen for the fullscreen flag-//
  else
    {//- This is a very dangerous loop to use, crash if we dont have an array-//
     int res_elements=0;
     printf("DEBUG:  Available Fullscreen Video Modes: \n");
     for(int i=0;modes[i];i++)
        {if((modes[i]->w) >=800 && (double(modes[i]->h)/double(modes[i]->w)) <= 1 )
           {res_elements++;
            printf("        %d x %d\n", modes[i]->w, modes[i]->h);
            resolution_add(modes[i]->w, modes[i]->h);
           }
        }
     printf("DEBUG:  Fullscreen Resolutions Available: %i \n",res_elements);
    }
 }
 
////////////////////////////////////////////////////////////////////////////////
//Name:    Set_Video
//Purpose: Sets up the SDL Window
////////////////////////////////////////////////////////////////////////////////
 void set_video(int res_x, int res_y, int bits, Uint32 flags)
 {printf("DEBUG:  Attempting to make SDL Window at %i x %i, %i bit depth \n",res_x, res_y, bits);
  screen=SDL_SetVideoMode(res_x, res_y, bits, flags);
  if(screen==NULL)
    {printf("CRITICAL EXISTANCE FAILURE: Unable to Create SDL Window \n");
     revert_defaults();
    }
  else
    {printf("DEBUG:  SDL Window Created Sucessfully \n");
     write_config();
    }
 }
////////////////////////////////////////////////////////////////////////////////
 public:
////////////////////////////////////////////////////////////////////////////////
 bool done;
 SDL_Surface *screen;
////////////////////////////////////////////////////////////////////////////////
//Name:    Get Resolution List
//Purpose: Returns the pointer to the resolution linked list
////////////////////////////////////////////////////////////////////////////////
resolution *get_resolution_list()
{return first;
}

////////////////////////////////////////////////////////////////////////////////
//Name:    Change Resolution
//Purpose: Change the resolution
////////////////////////////////////////////////////////////////////////////////
 void change_resolution(int res_x, int res_y, int bits, Uint32 flags)
 {resolution_x=res_x;
  resolution_y=res_y;
  color_depth=bits;
  fullscreen=flags & SDL_FULLSCREEN;
  //--//
  set_video(res_x, res_y, bits, flags);
 }
  
////////////////////////////////////////////////////////////////////////////////
//Name:    Init 
//Purpose: Initialize SDL and our main surface
////////////////////////////////////////////////////////////////////////////////
 void init(Uint32 init_flags)
 {if(SDL_Init(init_flags)<0)
    {printf("CRITICAL EXISTANCE FAILURE: Unable to Init SDL \n");
    }
  //--//
  fullscreen_check();
  if(read_config()==false)
    {printf("DEBUG:  Setting Window Title \n");
     SDL_WM_SetCaption(WINDOW_TEXT, NULL);  
     return;
    }
  //--//
  if(fullscreen==false)
    {set_video(resolution_x, resolution_y, color_depth, SDL_SWSURFACE);
    }
  //--//
  else
    {set_video(resolution_x, resolution_y, color_depth, SDL_SWSURFACE|SDL_FULLSCREEN);
    }
  printf("DEBUG:  Setting Window Title \n");
  SDL_WM_SetCaption(WINDOW_TEXT, NULL);  
 }
////////////////////////////////////////////////////////////////////////////////
}sdl;
