////////////////////////////////////////////////////////////////////////////////
//Name:    Surface
//Purpose: Base Surface Functions
////////////////////////////////////////////////////////////////////////////////
class surface
{
////////////////////////////////////////////////////////////////////////////////
 public:
////////////////////////////////////////////////////////////////////////////////
//Name:    Get Pixel
//Purpose: Gets specific pixel data from a surface
////////////////////////////////////////////////////////////////////////////////
 Uint32 get_pixel(SDL_Surface *image, int x, int y)
 {int pixel_bytes = image->format->BytesPerPixel;
  Uint8 *pixel = (Uint8 *)image->pixels + y * image->pitch + x * pixel_bytes;
  //-8 Bit surface-//
  if(pixel_bytes==1)
    {return *pixel;
    }
  //-16 bit surface-//
  else if(pixel_bytes==2)
    {printf("WARNING: Using a 16-bit image - surface->get_pixel\n");
     return *(Uint16 *)pixel;
    }
  //-24 bit surface-//
  else if(pixel_bytes==3)
    {printf("WARNING: Using a 24-bit image - surface->get_pixel\n");
     if(SDL_BYTEORDER == SDL_BIG_ENDIAN)
       {return pixel[0] << 16 | pixel[1] << 8 | pixel[2];
       }
     else
       {return pixel[0] | pixel[1] << 8 | pixel[2] << 16;
       }
    }
  //-32 bit surface-//
  else if(pixel_bytes==4)
    {return *(Uint32 *)pixel;
    }
  //-Hopefully Impossible-//
  else
    {printf("ERROR: Surface.GetPixel -> Nonstandard pixel format\n");
     return 0;
    }
  //--//
 }
 
////////////////////////////////////////////////////////////////////////////////
//Name:    Set Pixel
//Purpose: Sets a pixel in a surface
////////////////////////////////////////////////////////////////////////////////
 Uint32 set_pixel(SDL_Surface *image, int x, int y, Uint32 pixel)
 {int pixel_bytes = image->format->BytesPerPixel;
  /* Here p is the address to the pixel we want to set */
  Uint8 *pixel_edit = (Uint8 *)image->pixels + y * image->pitch + x * pixel_bytes;
  //-8 bit surface-//
  if(pixel_bytes==1)
    {*pixel_edit = pixel;
    }
  //-16 bit surface-//
  else if(pixel_bytes==2)
    {printf("WARNING: Using a 16-bit image - surface->set_pixel\n");
     *(Uint16 *)pixel_edit = pixel;
    }
  //-24 bit surface-//
  else if(pixel_bytes==3)
    {printf("WARNING: Using a 24-bit image - surface->set_pixel\n");
     if(SDL_BYTEORDER == SDL_BIG_ENDIAN) 
       {pixel_edit[0] = (pixel >> 16) & 0xff;
        pixel_edit[1] = (pixel >> 8) & 0xff;
        pixel_edit[2] = pixel & 0xff;
       }
     else 
       {pixel_edit[0] = pixel & 0xff;
        pixel_edit[1] = (pixel >> 8) & 0xff;
        pixel_edit[2] = (pixel >> 16) & 0xff;
       }
    }
  //-32 bit surface-//
  else if(pixel_bytes==4)
    {*(Uint32 *)pixel_edit = pixel;
    }
  //-Hopefully Impossible-//
  else  
    {printf("ERROR: Surface.SetPixel -> Nonstandard pixel format\n");
    } 
  //--//      
 }
 
////////////////////////////////////////////////////////////////////////////////
//Name:    Draw
//Purpose: Blit a complete surface to another surface
////////////////////////////////////////////////////////////////////////////////
 void draw(SDL_Surface *blit, SDL_Surface *blit_dest, int x, int y)
 {SDL_Rect dest = {x, y, 0, 0};
  if(SDL_BlitSurface(blit, NULL, blit_dest, &dest)!=0)
    {printf("ERROR: Surface.Draw -> Unable to blit surface \n");
    }
 }

////////////////////////////////////////////////////////////////////////////////
//Name:    Draw Partial
//Purpose: Blit part of a surface to another surface
//Notes:   Shipwrecked Clip Rect of a Clipped Rect Shipwreck!
////////////////////////////////////////////////////////////////////////////////
 void draw_partial(SDL_Surface *blit, int clip_w, int clip_h, int clip_x, int clip_y, SDL_Surface *blit_dest, int dest_x, int dest_y)
 {SDL_Rect dest_rect = {dest_x, dest_y, 0, 0};
  SDL_Rect clip_rect = {clip_x, clip_y, clip_w, clip_h};
  if(SDL_BlitSurface(blit, &clip_rect, blit_dest, &dest_rect)!=0)
    {printf("ERROR: Surface.DrawPartial -> Unable to blit surface \n");
    }
 }

////////////////////////////////////////////////////////////////////////////////
}surface;
